var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import React, { createContext } from 'react';
import { Engine as BabylonJSEngine, Observable, } from '@babylonjs/core';
// TODO: build a fallback mechanism when typeof React.createContext !== 'function'
// this will allow (16.0 <= react versions < 16.3) to work.
export var BabylonJSContext = createContext({
    engine: null,
    canvas: null
});
export function withBabylonJS(Component) {
    return function BoundComponent(props) {
        return (React.createElement(BabylonJSContext.Consumer, null, function (ctx) { return React.createElement(Component, __assign({}, props, { babylonJSContext: ctx })); }));
    };
}
var Engine = /** @class */ (function (_super) {
    __extends(Engine, _super);
    function Engine(props) {
        var _this = _super.call(this, props) || this;
        _this.engine = null;
        _this.canvas = null;
        _this.onBeforeRenderLoopObservable = new Observable();
        _this.onEndRenderLoopObservable = new Observable();
        _this.onCanvasRef = function (c) {
            // We are not using the react.createPortal(...), as it adds a ReactDOM dependency, but also
            // it was not flowing the context through to HOCs properly.
            if (_this.props.portalCanvas) {
                _this.canvas = document.getElementById('portal-canvas');
                console.error('set canvas', _this.canvas);
            }
            else {
                if (c) { // null when called from unmountComponent()
                    // c.addEventListener('mouseover', this.focus)
                    // c.addEventListener('mouseout', this.blur)
                    _this.canvas = c;
                }
            }
            // console.error('onCanvas:', c); // trying to diagnose why HMR keep rebuilding entire Scene!  Look at ProxyComponent v4.
        };
        _this.onResizeWindow = function () {
            if (_this.engine) {
                _this.engine.resize();
            }
        };
        _this.state = {
            canRender: false
        };
        return _this;
    }
    Engine.prototype.componentDidMount = function () {
        var _this = this;
        this.engine = new BabylonJSEngine(this.canvas, this.props.antialias === true ? true : false, // default false
        this.props.engineOptions, this.props.adaptToDeviceRatio === true ? true : false // default false
        );
        this.engine.runRenderLoop(function () {
            if (_this.onBeforeRenderLoopObservable.hasObservers()) {
                _this.onBeforeRenderLoopObservable.notifyObservers(_this.engine);
            }
            _this.engine.scenes.forEach(function (scene) {
                scene.render();
            });
            if (_this.onEndRenderLoopObservable.hasObservers()) {
                _this.onEndRenderLoopObservable.notifyObservers(_this.engine);
            }
        });
        this.engine.onContextLostObservable.add(function (eventData) {
            console.log('context loss observable from Engine: ', eventData);
        });
        window.addEventListener('resize', this.onResizeWindow);
        this.setState({ canRender: true });
    };
    Engine.prototype.componentWillUnmount = function () {
        window.removeEventListener('resize', this.onResizeWindow);
        if (this.engine !== null) {
            this.engine.dispose();
            this.engine = null;
        }
    };
    Engine.prototype.render = function () {
        if (this.state.canRender === false && (this.props.noSSR !== undefined && this.props.noSSR !== false)) {
            if (typeof this.props.noSSR === 'boolean') {
                return null;
            }
            return this.props.noSSR;
        }
        var _a = this.props, touchActionNone = _a.touchActionNone, width = _a.width, height = _a.height, canvasStyle = _a.canvasStyle, canvasId = _a.canvasId, rest = __rest(_a, ["touchActionNone", "width", "height", "canvasStyle", "canvasId"]);
        var opts = {};
        if (touchActionNone !== false) {
            opts['touch-action'] = 'none';
        }
        if (width !== undefined && height !== undefined) {
            opts.width = width;
            opts.height = height;
        }
        if (canvasId) {
            opts.id = canvasId;
        }
        if (canvasStyle) {
            opts.style = canvasStyle;
        }
        // TODO: this.props.portalCanvas does not need to render a canvas.
        return React.createElement(BabylonJSContext.Provider, { value: { engine: this.engine, canvas: this.canvas } },
            React.createElement("canvas", __assign({}, opts, { ref: this.onCanvasRef }), this.engine !== null &&
                this.props.children));
    };
    return Engine;
}(React.Component));
export default Engine;
//# sourceMappingURL=Engine.js.map