var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
import * as BABYLONEXT from "./extensions";
import * as GENERATED from './generatedCode';
import * as CUSTOM_HOSTS from "./customHosts";
import { FiberModel, LoadedModel } from "./model";
import { CreationType } from "./codeGenerationDescriptors";
import { applyUpdateToInstance, applyInitialPropsToInstance } from "./UpdateInstance";
function createCreatedInstance(className, hostInstance, propsHandlers, metadata, customProps, lifecycleListener) {
    var createdMetadata = metadata;
    // TODO: move how this is generated as a boolean to a metadata on objects themselves (and the next 3 lines!).
    if (propsHandlers.isTargetable === true) {
        createdMetadata.isTargetable = true;
    }
    return {
        hostInstance: hostInstance,
        metadata: createdMetadata,
        parent: null,
        children: [],
        propsHandlers: propsHandlers,
        lifecycleListener: lifecycleListener,
        customProps: customProps
    };
}
/**
 * remove instance's children recursively
 *
 * @param parentInstance
 * @param child
 */
function removeChild(parentInstance, child) {
    if (child) {
        var hostInstance = child.hostInstance;
        if (child.lifecycleListener) {
            child.lifecycleListener.onUnmount();
        }
        if (child.metadata.isNode) {
            hostInstance.parent = null;
        }
        else if (child.metadata.isGUI3DControl === true) {
            console.error("3D remove control not implemented.");
        }
        if (parentInstance.metadata && parentInstance.metadata.isGUI2DControl === true && child.metadata.isGUI2DControl === true) {
            // NOTE: the if statement should be || and we may need to walk the tree to remove.
            parentInstance.hostInstance.removeControl(child.hostInstance);
        }
        if (child.children) {
            removeRecursive(child.children, child);
        }
        if (typeof child.hostInstance.dispose === "function") {
            hostInstance.dispose(); // TODO: Consider adding metadata/descriptors as some dispose methods have optional args.
        }
        // fix: old version rootInstance.children was not cleaned, so the children array maybe huge over time
        parentInstance.children = parentInstance.children.filter(function (ci) { return ci !== child; });
        child.parent = null;
    }
}
/**
 * remove child recursive
 */
function removeRecursive(array, parent, clone) {
    if (clone === void 0) { clone = false; }
    if (array) {
        var target = clone ? __spreadArrays(array) : array;
        target.forEach(function (child) { return removeChild(parent, child); });
    }
}
var ReactBabylonJSHostConfig = {
    // This has the reconciler include in call chain ie: appendChild, removeChild
    get supportsMutation() {
        return true;
    },
    now: function () {
        return Date.now(); // TODO: use performance.now
    },
    // multiple renderers concurrently render using the same context objects. E.g. React DOM and React ART on the
    // same page. DOM is the primary renderer; ART is the secondary renderer.
    get isPrimaryRenderer() {
        return false;
    },
    get supportsPersistence() {
        return false;
    },
    // TODO: see if this will allow ie: improved HMR support.  Need to implement a lot of currently optional methods.
    get supportsHydration() {
        return false;
    },
    // this enables refs
    getPublicInstance: function (instance) {
        return instance;
    },
    getRootHostContext: function (rootContainerInstance) {
        // this is the context you pass to your chilren, as parameter 'parentHostContext' from "root".
        // So, opportunity to share context here via HostConfig further up tree.
        return {
            canvas: rootContainerInstance.canvas,
            engine: rootContainerInstance.engine,
            scene: rootContainerInstance.scene,
            rootInstance: {
                hostInstance: undefined,
                metadata: {
                    className: "rootContainer",
                    namespace: "ignore"
                },
                parent: null,
                children: [],
                customProps: {}
            }
        };
    },
    // this is the context you pass down to children.  without this root will not be available to attach to in appendChildToContainer.
    getChildHostContext: function (parentHostContext, type, rootContainerInstance) {
        return parentHostContext;
    },
    prepareUpdate: function (instance, type, oldProps, newProps, rootContainerInstance, hostContext) {
        if (!instance || (instance.metadata && instance.metadata.customType === true)) {
            return null;
        }
        var updatePayload = [];
        // Only custom types will not have a fiber object to handle props changes
        instance.propsHandlers.getPropsHandlers().forEach(function (propHandler) {
            var handlerUpdates = propHandler.getPropertyUpdates(oldProps, newProps);
            if (handlerUpdates !== null) {
                updatePayload.push.apply(updatePayload, handlerUpdates);
            }
        });
        return updatePayload.length === 0 ? null : updatePayload;
    },
    /**
     * This is called when something is dynamically added to root (not on initial appendChildToContainer)
     */
    insertInContainerBefore: function (container, child, beforeChild) {
        // same implementation as appendChildToContainer
        if (child) {
            // doubly link child to root.  we don't care about order - ie: 'beforeChild'
            container.rootInstance.children.push(child);
            child.parent = container.rootInstance;
        }
        else {
            console.error("insertInContainerBefore. No child:", child);
        }
    },
    createInstance: function (type, props, rootContainerInstance, hostContext, internalInstanceHandle) {
        var _a, _b;
        // TODO: Make a registry like React Native host config or just build a map in /customHosts/index.ts.
        var customTypes = [CUSTOM_HOSTS.HostWithEvents];
        // TODO: Check source for difference between hostContext and rootContainerInstance.
        var canvas = rootContainerInstance.canvas, engine = rootContainerInstance.engine, scene = rootContainerInstance.scene;
        if (customTypes.indexOf(type) !== -1) {
            var metadata_1 = __assign({ className: type, customType: true }, props.metadata);
            var createdInstance = {
                hostInstance: null,
                metadata: metadata_1,
                parent: null,
                children: [],
                propsHandlers: undefined,
                customProps: {},
                lifecycleListener: new CUSTOM_HOSTS[type + "Fiber"](scene, engine, props)
            };
            // onCreated and other lifecycle hooks are not called for built-in host
            return createdInstance;
        }
        // so far this is the only non-babylonJS host component, but otherwise a more generic solution will be needed:
        if (type.toLowerCase() === "model") {
            var createdInstance = {
                hostInstance: new LoadedModel() /* this is reassigned in Lifecycle Listener */,
                metadata: {
                    className: "Model"
                },
                parent: null,
                children: [],
                propsHandlers: new FiberModel(),
                lifecycleListener: new CUSTOM_HOSTS.ModelLifecycleListener(scene /* should always be available */, props),
                customProps: {}
            };
            // onCreated and other lifecycle hooks are not called for built-in host
            return createdInstance;
        }
        // some types (ie: button) are called 'babylonjs-button'.
        var underlyingClassName = GENERATED.intrinsicClassMap[type] || type;
        var classDefinition = GENERATED["Fiber" + underlyingClassName];
        if (classDefinition === undefined) {
            throw new Error("Cannot generate type '" + type + "/" + underlyingClassName + "' inside 'react-babylonjs' (ie: no DOM rendering on HTMLCanvas)");
        }
        var createInfoArgs = classDefinition.CreateInfo;
        var metadata = classDefinition.Metadata;
        // console.log(`creating: ${createInfoArgs.namespace}.${type}`)
        var generatedParameters = createInfoArgs.parameters;
        // console.log("generated params:", generatedParameters)
        var args = generatedParameters.map(function (generatedParameter) {
            if (Array.isArray(generatedParameter.type)) {
                // TODO: if all props are missing, warn if main prop (ie: options) is required.
                var newParameter_1 = {};
                generatedParameter.type.forEach(function (subParameter) {
                    var subPropValue = props[subParameter.name];
                    if (subPropValue === undefined && subParameter.optional === false && generatedParameter.optional === false) {
                        console.warn("Missing a required secondary property:", subParameter.name);
                    }
                    else {
                        newParameter_1[subParameter.name] = subPropValue;
                    }
                });
                return newParameter_1;
            }
            else {
                var value = props[generatedParameter.name];
                if (value === undefined) {
                    // NOTE: we removed the hosted Scene component, which needs (generatedParameter.type == "BabylonjsCoreEngine")
                    // SceneOrEngine type is Scene
                    if (generatedParameter.type.includes("BabylonjsCoreScene") || (generatedParameter.type === "any" && generatedParameter.name === "scene")) {
                        // MeshBuild.createSphere(name: string, options: {...}, scene: any)
                        // console.log('Assigning scene to:', type, generatedParameter)
                        value = scene;
                    }
                    else if (generatedParameter.optional === false) {
                        console.warn("required parameter for " + type + " unassigned -> " + generatedParameter.name + ":" + generatedParameter.type);
                    }
                }
                if (value === undefined && generatedParameter.optional === false) {
                    console.warn("On " + type + " you are missing a non-optional parameter '" + generatedParameter.name + "' of type '" + generatedParameter.type + "'");
                }
                return value;
            }
        });
        var babylonObject = undefined;
        if (createInfoArgs.creationType === CreationType.FactoryMethod) {
            // console.warn(`creating from Factory: ${createInfoArgs.libraryLocation}.${createInfoArgs.factoryMethod}(...args).  args:`, args)
            babylonObject = (_a = GENERATED.babylonClassFactory(createInfoArgs.libraryLocation))[createInfoArgs.factoryMethod].apply(_a, args);
        }
        else {
            if (metadata.delayCreation !== true) {
                if (createInfoArgs.namespace.startsWith('@babylonjs/')) {
                    var clazz = GENERATED.babylonClassFactory(type);
                    if (clazz === undefined) {
                        throw new Error("Cannot generate '" + type + "' (react-babylonjs):");
                    }
                    babylonObject = new (clazz.bind.apply(clazz, __spreadArrays([void 0], args)))();
                }
                else if (createInfoArgs.namespace.startsWith('./extensions/')) {
                    var extClassName = GENERATED.intrinsicClassMap[type] || type;
                    babylonObject = new ((_b = BABYLONEXT[extClassName]).bind.apply(_b, __spreadArrays([void 0], args)))();
                }
                else {
                    console.error("metadata defines (or does not) a namespace that is known", createInfoArgs.namespace);
                }
            }
        }
        // Developer accessible lifecycle phase.  ie: access propery/method exposed in props.
        if (typeof props.onCreated === "function") {
            props.onCreated(babylonObject);
        }
        var fiberObject = new GENERATED["Fiber" + underlyingClassName]();
        var lifecycleListener = undefined;
        // TODO: Add these to just the 'type' of component they apply to.
        var customProps = {
            childrenAsContent: props.childrenAsContent === true,
            createForParentMesh: props.createForParentMesh === true,
            onControlAdded: typeof props.onControlAdded === "function" ? props.onControlAdded : undefined,
            connectControlNames: props.connectControlNames,
            defaultKeyboard: props.defaultKeyboard === true,
            linkToTransformNodeByName: props.linkToTransformNodeByName,
            shadowCasters: props.shadowCasters,
            shadowCastersExcluding: props.shadowCastersExcluding,
            attachToMeshesByName: props.attachToMeshesByName,
            assignTo: props.assignTo // here a lifecycle listener can dynamically attach to another property (ie: Mesh to DynamicTerrain -> 'mesh.material')
        };
        // Consider these being dynamically attached to a list, much like PropsHandlers<T>
        if (metadata.isMaterial === true) {
            lifecycleListener = new CUSTOM_HOSTS.MaterialsLifecycleListener();
        }
        else if (metadata.isGUI3DControl === true) {
            lifecycleListener = new CUSTOM_HOSTS.GUI3DControlLifecycleListener(scene);
        }
        else if (metadata.isGUI2DControl === true) {
            lifecycleListener = new CUSTOM_HOSTS.GUI2DControlLifecycleListener();
        }
        else if (metadata.isTexture === true) {
            lifecycleListener = new CUSTOM_HOSTS.TexturesLifecycleListener();
        }
        else if (metadata.isCamera === true) {
            lifecycleListener = new CUSTOM_HOSTS.CameraLifecycleListener(scene, props, canvas);
        }
        else if (metadata.isNode) {
            lifecycleListener = new CUSTOM_HOSTS.NodeLifecycleListener();
        }
        else if (metadata.isBehavior) {
            lifecycleListener = new CUSTOM_HOSTS.BehaviorLifecycleListener();
        }
        // here we dynamically assign listeners for specific types.
        // TODO: need to double-check because we are using 'camelCase'
        if (CUSTOM_HOSTS[underlyingClassName + "LifecycleListener"] !== undefined) {
            lifecycleListener = new CUSTOM_HOSTS[underlyingClassName + "LifecycleListener"](scene, props);
        }
        var createdReference = createCreatedInstance(underlyingClassName, babylonObject, fiberObject, metadata, customProps, lifecycleListener);
        if (lifecycleListener && lifecycleListener.onCreated) {
            lifecycleListener.onCreated(createdReference, scene);
        }
        // Here we dynamically attach known props handlers.  Will be adding more in code generation for GUI - also for lifecycle mgmt.
        if (createdReference.metadata && createdReference.metadata.isTargetable === true) {
            fiberObject.addPropsHandler(new CUSTOM_HOSTS.TargetPropsHandler(scene));
        }
        if (metadata.delayCreation !== true) {
            applyInitialPropsToInstance(createdReference, props);
        }
        else {
            createdReference.deferredCreationProps = props;
        }
        return createdReference;
    },
    shouldDeprioritizeSubtree: function (type, props) {
        return false;
    },
    hideInstance: function (instance) { },
    unhideInstance: function (instance, props) { },
    createTextInstance: function () {
        return undefined;
    },
    scheduleDeferredCallback: function (callback, opts) {
        return window.requestIdleCallback(callback, opts); // ReactDOMHostConfig has: unstable_scheduleCallback as scheduleDeferredCallback
    },
    cancelDeferredCallback: function (handle) {
        return window.cancelIdleCallback(handle);
    },
    setTimeout: function (handler, timeout) {
        return window.setTimeout(handler);
    },
    clearTimeout: function (handle) {
        window.clearTimeout(handle);
    },
    // https://github.com/facebook/react/blob/master/packages/react-dom/src/client/ReactDOMHostConfig.js#L288
    noTimeout: -1,
    // Called based on return value of: finalizeInitialChildren.  in-memory render tree created, but not yet attached.
    prepareForCommit: function (containerInfo) { },
    // Called after the in-memory tree has been committed (ie: after attaching again to root element)
    resetAfterCommit: function (containerInfo) { },
    appendInitialChild: function (parent, child) {
        // Here we are traversing downwards.  Beyond parent has not been initialized, but all children have been.
        if (parent) {
            if (!child) {
                console.error('undefined child', parent);
            }
            else {
                // doubly linking child to parent
                parent.children.push(child); // TODO: need to remove from children as well when removing.
                child.parent = parent;
            }
        }
        if (child && child.lifecycleListener && child.lifecycleListener.onParented) {
            child.lifecycleListener.onParented(parent, child);
        }
        if (parent && parent.lifecycleListener && parent.lifecycleListener.onChildAdded) {
            parent.lifecycleListener.onChildAdded(child, parent);
        }
    },
    // TODO: refactor with appendInitialChild
    appendChild: function (parent, child) {
        parent.children.push(child);
        child.parent = parent;
        if (child && child.lifecycleListener && child.lifecycleListener.onParented) {
            child.lifecycleListener.onParented(parent, child);
        }
        if (parent && parent.lifecycleListener && parent.lifecycleListener.onChildAdded) {
            parent.lifecycleListener.onChildAdded(child, parent);
        }
    },
    canHydrateInstance: function (instance, type, props) {
        // console.log("canHydrateInstance", instance, type, props)
        return null;
    },
    // The parent of this node has not yet been instantiated.  The reconciler will continue by calling:
    // createInstance → appendInitialChild → finalizeInitialChildren on the parent.
    // When that has reached the top of the recursion tree (root), then prepareForCommit() will be called.
    // NOTE: All children of this component, however, HAVE been initialized.
    finalizeInitialChildren: function () {
        return true; /* callCommitMountForThisInstance */
    },
    commitMount: function (instance, type, newProps, internalInstanceHandle) {
        if (instance && instance.lifecycleListener && instance.lifecycleListener.onMount) {
            instance.lifecycleListener.onMount(instance);
        }
    },
    // NOTE: only called if supportsMutation = true;
    // ReactDOM uses this for attaching child nodes to root DOM.  For us we want to link the all parts of tree together for tree crawling.
    // same implementation as insertInContainerBefore
    appendChildToContainer: function (container, child) {
        if (child) {
            // doubly link child to root
            container.rootInstance.children.push(child);
            child.parent = container.rootInstance;
        }
    },
    commitUpdate: function (instance, updatePayload, type /* old + new props are extra params here */) {
        if (updatePayload !== null) {
            updatePayload.forEach(function (update) {
                applyUpdateToInstance(instance.hostInstance, update, type);
            });
        }
    },
    removeChildFromContainer: function (container, child) {
        /**
         * To fix two bugs when toggle meshes:
         * 1.  model's mesh can't be destroyed.
         * 2. `removeChildFromContainer()` only destroy babylon instance.
         * The model is rootAbstractMesh's and Parent's child.
         * `container.rootInstance.children` will be very large after few toggles.
         */
        if (child) {
            removeChild(container.rootInstance, child);
        }
    },
    removeChild: removeChild,
    insertBefore: function (parentInstance, child, beforeChild) {
    },
    // text-content nodes are not used.  treated as a leaf node.  children are not traversed.  calls methods like createTextInstance(...)
    shouldSetTextContent: function (type, props) {
        return false;
    }
};
export default ReactBabylonJSHostConfig;
//# sourceMappingURL=ReactBabylonJSHostConfig.js.map