/**
 * react-babylonjs
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE.txt file in the root directory of this source tree.
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React, { createContext, useContext, useEffect, useRef, useState, } from 'react';
import ReactReconciler from "react-reconciler";
import { BabylonJSContext, withBabylonJS } from './Engine';
import { PointerEventTypes, Scene as BabylonJSScene } from '@babylonjs/core';
import { applyUpdateToInstance } from "./UpdateInstance";
import ReactBabylonJSHostConfig from './ReactBabylonJSHostConfig';
import { FiberScenePropsHandler } from './generatedCode';
// TODO: build a fallback mechanism when typeof React.createContext !== 'function'
export var SceneContext = createContext({
    engine: null,
    canvas: null,
    scene: null,
    sceneReady: false
});
export var useBabylonEngine = function () { return useContext(SceneContext).engine; };
export var useBabylonScene = function () { return useContext(SceneContext).scene; };
export var useBabylonCanvas = function () { return useContext(SceneContext).canvas; };
export function withScene(Component) {
    return function BoundComponent(props) {
        return (React.createElement(SceneContext.Consumer, null, function (ctx) { return React.createElement(Component, __assign({}, props, { sceneContext: ctx })); }));
    };
}
var usePrevious = function (value) {
    var ref = useRef();
    useEffect(function () {
        ref.current = value;
    }, [value]);
    return ref.current;
};
var Scene = function (props, context) {
    var engine = useContext(BabylonJSContext).engine;
    var propsHandler = useState(new FiberScenePropsHandler())[0];
    var _a = useState(false), sceneReady = _a[0], setSceneReady = _a[1];
    var _b = useState(null), scene = _b[0], setScene = _b[1];
    var _c = useState(null), fiberRoot = _c[0], setFiberRoot = _c[1];
    // TODO: make this strongly typed
    var _d = useState(null), renderer = _d[0], setRenderer = _d[1];
    var prevPropsRef = useRef({});
    // initialize babylon scene
    useEffect(function () {
        // const onSceneReady = (eventData: BabylonJSScene, eventState: EventState) => {
        //   setSceneReady(true);
        // }
        var scene = new BabylonJSScene(engine, props.sceneOptions);
        // const onReadyObservable: Nullable<Observer<BabylonJSScene>> = scene.onReadyObservable.add(onSceneReady);
        if (scene.isReady()) {
            // scene.onReadyObservable.remove(onReadyObservable);
            setSceneReady(true);
        }
        else {
            console.error('Scene is not ready. Report issue in react-babylonjs repo');
        }
        setScene(scene);
        var isAsync = false; // Disables experimental async rendering
        var container = {
            engine: props.babylonJSContext.engine,
            canvas: props.babylonJSContext.canvas,
            scene: scene,
            rootInstance: {
                hostInstance: null,
                children: [],
                parent: null,
                metadata: {
                    className: "root"
                },
                customProps: {}
            }
        };
        var renderer = ReactReconciler(ReactBabylonJSHostConfig);
        setRenderer(renderer);
        var fiberRoot = renderer.createContainer(container, isAsync, false /* hydrate true == better HMR? */);
        setFiberRoot(fiberRoot);
        renderer.injectIntoDevTools({
            bundleType: process.env.NODE_ENV === 'production' ? 0 : 1,
            version: '2.0.0',
            rendererPackageName: 'react-babylonjs'
        });
        var pointerDownObservable = scene.onPointerObservable.add(function (evt) {
            if (typeof props.onScenePointerDown === 'function') {
                props.onScenePointerDown(evt, scene);
            }
            if (evt && evt.pickInfo && evt.pickInfo.hit && evt.pickInfo.pickedMesh) {
                var mesh = evt.pickInfo.pickedMesh;
                if (typeof props.onMeshPicked === 'function') {
                    props.onMeshPicked(mesh, scene);
                }
                else {
                    // console.log('onMeshPicked not being called')
                }
            }
        }, PointerEventTypes.POINTERDOWN);
        // can only be assigned on init
        var pointerUpObservable = null;
        if (typeof props.onScenePointerUp === 'function') {
            pointerUpObservable = scene.onPointerObservable.add(function (evt) {
                props.onScenePointerUp(evt, scene);
            }, PointerEventTypes.POINTERUP);
        }
        ;
        // can only be assigned on init
        var pointerMoveObservable = null;
        if (typeof props.onScenePointerMove === 'function') {
            pointerMoveObservable = scene.onPointerObservable.add(function (evt) {
                props.onScenePointerMove(evt, scene);
            }, PointerEventTypes.POINTERMOVE);
        }
        ;
        if (typeof props.onSceneMount === 'function') {
            props.onSceneMount({
                scene: scene,
                canvas: scene.getEngine().getRenderingCanvas()
            });
            // TODO: console.error if canvas is not attached. runRenderLoop() is expected to be part of onSceneMount().
        }
        // TODO: change enable physics to 'usePhysics' taking an object with a Vector3 and 'any'.
        // NOTE: must be enabled for updating container (cannot add impostors w/o physics enabled)
        if (Array.isArray(props.enablePhysics)) {
            scene.enablePhysics(props.enablePhysics[0], props.enablePhysics[1]);
        }
        // update the root Container
        renderer.updateContainer(React.createElement(SceneContext.Provider, { value: {
                engine: props.babylonJSContext.engine,
                canvas: props.babylonJSContext.canvas,
                scene: scene,
                sceneReady: sceneReady
            } }, props.children), fiberRoot, undefined, function () {
        });
        return function () {
            if (pointerDownObservable) {
                scene.onPointerObservable.remove(pointerDownObservable);
            }
            if (pointerUpObservable) {
                scene.onPointerObservable.remove(pointerUpObservable);
            }
            if (pointerMoveObservable) {
                scene.onPointerObservable.remove(pointerMoveObservable);
            }
            if (scene.isDisposed === false) {
                scene.dispose();
            }
        };
    }, [ /* no deps, so called only on un/mount */]);
    // update babylon scene
    useEffect(function () {
        if (engine === null || scene === null || renderer === null) {
            return;
        }
        var prevProps = prevPropsRef.current;
        var updates = propsHandler.getPropertyUpdates(prevProps, props);
        if (updates !== null) {
            updates.forEach(function (propertyUpdate) {
                applyUpdateToInstance(scene, propertyUpdate, 'scene');
            });
        }
        renderer.updateContainer(React.createElement(SceneContext.Provider, { value: {
                engine: props.babylonJSContext.engine,
                canvas: props.babylonJSContext.canvas,
                scene: scene,
                sceneReady: sceneReady
            } }, props.children), fiberRoot, undefined, function () {
        });
        prevPropsRef.current = props;
    });
    return null;
};
export default withBabylonJS(Scene);
//# sourceMappingURL=Scene.js.map