var GUI3DControlLifecycleListener = /** @class */ (function () {
    function GUI3DControlLifecycleListener(scene) {
        this.scene = scene;
    }
    GUI3DControlLifecycleListener.prototype.onParented = function (parent, child) { };
    GUI3DControlLifecycleListener.prototype.onChildAdded = function (child, parent) { };
    GUI3DControlLifecycleListener.prototype.onMount = function (instance) {
        if (instance.state && instance.state.added === true) {
            return;
        }
        var addedParent = null;
        var tmp = instance.parent;
        while (tmp) {
            if (tmp.metadata.isGUI3DControl === true) {
                if (tmp.state && tmp.state.added === true) {
                    addedParent = tmp;
                    break;
                }
            }
            tmp = tmp.parent;
        }
        if (addedParent) {
            this.addControls(addedParent, addedParent);
        }
    };
    GUI3DControlLifecycleListener.prototype.addControls = function (instance, last3DGuiControl) {
        var _this = this;
        var last3d = instance.metadata.isGUI3DControl === true ? instance : last3DGuiControl;
        instance.children.forEach(function (child) {
            if (last3d.customProps.childrenAsContent === true) {
                last3d.hostInstance.content = child.hostInstance; // child.hostInstance will be GUI.Control (ie: 2D)
                child.state = { added: true, content: true };
            }
            else if (child.metadata.isGUI3DControl === true) {
                last3d.hostInstance.addControl(child.hostInstance);
                // NOTE: this must be called after .addControl(child.hostInstance).
                if (child.customProps.linkToTransformNodeByName) {
                    var toLinkTo = _this.scene.getMeshByName(child.customProps.linkToTransformNodeByName);
                    if (toLinkTo !== null) {
                        child.hostInstance.linkToTransformNode(toLinkTo);
                        // toLinkTo.computeWorldMatrix()
                    }
                    else {
                        console.error("linkToTransformNode cannot find Mesh:", instance.customProps.linkToTransformNodeByName, " and does not have a scene listener for added meshes." + " Declare Mesh earlier or add an issue on github (TransformNode does not work).");
                    }
                }
                child.state = { added: true };
            }
        });
        if (instance.children.length > 0) {
            instance.children.forEach(function (child) {
                if (!child.state || child.state.content !== true) {
                    if (child.state && child.state.added === true && child.customProps.onControlAdded) {
                        child.customProps.onControlAdded(child);
                    }
                    _this.addControls(child, last3d);
                }
            });
        }
    };
    GUI3DControlLifecycleListener.prototype.onUnmount = function () { };
    return GUI3DControlLifecycleListener;
}());
export default GUI3DControlLifecycleListener;
//# sourceMappingURL=GUI3DControlLifecycleListener.js.map