var GUI3DManagerLifecycleListener = /** @class */ (function () {
    function GUI3DManagerLifecycleListener(scene) {
        this.scene = scene;
    }
    GUI3DManagerLifecycleListener.prototype.onParented = function (parent) { };
    GUI3DManagerLifecycleListener.prototype.onChildAdded = function (child) { };
    GUI3DManagerLifecycleListener.prototype.onMount = function (instance) {
        if (instance === undefined) {
            console.error('Missing instance');
            return;
        }
        this.addControls(instance, instance);
    };
    /**
     * We may have BoundComponents inbetween gui3d controls.
     */
    GUI3DManagerLifecycleListener.prototype.addControls = function (instance, last3DGuiControl) {
        var _this = this;
        // When there is a panel, it must be added before the children. Otherwise there is no UtilityLayer to attach to.
        // This project before 'react-reconciler' was added from parent up the tree.  'react-reconciler' wants to do the opposite.
        instance.children.forEach(function (child) {
            if (child.metadata.isGUI3DControl === true) {
                if (last3DGuiControl.customProps.childrenAsContent === true) {
                    last3DGuiControl.hostInstance.content = child.hostInstance;
                    child.state = { added: true, content: true };
                }
                else {
                    last3DGuiControl.hostInstance.addControl(child.hostInstance);
                    child.state = { added: true };
                    // NOTE: this must be called after .addControl(child.hostInstance).
                    if (child.customProps.linkToTransformNodeByName) {
                        var toLinkTo = _this.scene.getMeshByName(child.customProps.linkToTransformNodeByName);
                        if (toLinkTo !== null) {
                            child.hostInstance.linkToTransformNode(toLinkTo);
                        }
                        else {
                            console.error("linkToTransformNode cannot find ", instance.customProps.linkToTransformNodeByName, " and does not have a scene listener for added meshes.  Declare earlier or add an issue on github.");
                        }
                    }
                }
            }
            if (child.state && child.state.added === true && child.customProps.onControlAdded) {
                child.customProps.onControlAdded(child);
            }
            if (!child.state || child.state.content !== true) {
                var last3d = child.metadata.isGUI3DControl === true ? child : last3DGuiControl;
                _this.addControls(child, last3d);
            }
        });
        // Here we can now do a transform with an anchor point.
        console.error("transform gui3dmanager anchor missing");
    };
    GUI3DManagerLifecycleListener.prototype.onUnmount = function () { };
    return GUI3DManagerLifecycleListener;
}());
export default GUI3DManagerLifecycleListener;
//# sourceMappingURL=GUI3DManagerLifecycleListener.js.map