import { Vector3, BoundingInfo } from "@babylonjs/core";
import "@babylonjs/loaders";
export var LoaderStatus;
(function (LoaderStatus) {
    LoaderStatus[LoaderStatus["Loading"] = 0] = "Loading";
    LoaderStatus[LoaderStatus["Loaded"] = 1] = "Loaded";
    LoaderStatus[LoaderStatus["Error"] = 2] = "Error";
})(LoaderStatus || (LoaderStatus = {}));
var LoadedModel = /** @class */ (function () {
    function LoadedModel() {
    }
    Object.defineProperty(LoadedModel.prototype, "boundingInfo", {
        get: function () {
            if (!this.rootMesh) {
                return null;
            }
            // meshes are already parented to root mesh, so we do not need to look further.
            var min = null;
            var max = null;
            this.rootMesh.getChildMeshes().forEach(function (childMesh) {
                var _a = childMesh.getBoundingInfo().boundingBox, minimumWorld = _a.minimumWorld, maximumWorld = _a.maximumWorld;
                if (min === null) {
                    min = minimumWorld;
                }
                else {
                    min = Vector3.Minimize(min, minimumWorld);
                }
                if (max === null) {
                    max = maximumWorld;
                }
                else {
                    max = Vector3.Maximize(max, maximumWorld);
                }
            });
            if (min !== null && max != null) {
                return new BoundingInfo(min, max);
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Clean up all resources.
     */
    LoadedModel.prototype.dispose = function () {
        if (this.meshes) {
            this.meshes.forEach(function (mesh) {
                mesh.dispose(false /* not recursive */, true /* materials + textures */);
            });
            this.meshes = [];
        }
        if (this.particleSystems) {
            this.particleSystems.forEach(function (ps) {
                // ps.stop();
                ps.dispose();
            });
            this.particleSystems = [];
        }
        if (this.skeletons) {
            this.skeletons.forEach(function (skeleton) { return skeleton.dispose(); });
            this.skeletons = [];
        }
        if (this.animationGroups) {
            this.animationGroups.forEach(function (animationGroup) { return animationGroup.dispose(); });
            this.animationGroups = [];
        }
    };
    return LoadedModel;
}());
export { LoadedModel };
//# sourceMappingURL=LoadedModel.js.map