import { AbstractMesh, SceneLoader } from "@babylonjs/core";
import "@babylonjs/loaders";
import { applyUpdateToInstance } from "../UpdateInstance";
import { LoaderStatus, LoadedModel, ModelPropsHandler } from "../model";
/**
 * This listener has no underlying babylonjs type.  We are handling loading a 3D model with underlying mesh(es).
 */
var ModelLifecycleListener = /** @class */ (function () {
    function ModelLifecycleListener(scene, props) {
        this.parent = null;
        this.props = props;
        this.scene = scene;
    }
    ModelLifecycleListener.prototype.onParented = function (parent, child) {
        this.parent = parent;
    };
    ModelLifecycleListener.prototype.onChildAdded = function (child, parent) { };
    ModelLifecycleListener.prototype.onMount = function (instance) {
        var _this = this;
        var loadedModel = new LoadedModel();
        loadedModel.status = LoaderStatus.Loading;
        instance.hostInstance = loadedModel;
        var loader = SceneLoader.ImportMesh(undefined, this.props.rootUrl, this.props.sceneFilename, this.scene, function (meshes, particleSystems, skeletons, animationGroups) {
            var _a;
            loadedModel.rootMesh = new AbstractMesh(_this.props.sceneFilename + "-root-model");
            loadedModel.rootMesh.alwaysSelectAsActiveMesh = true;
            loadedModel.rootMesh.parent = (_a = _this.parent) === null || _a === void 0 ? void 0 : _a.hostInstance;
            loadedModel.meshes = [];
            meshes.forEach(function (mesh) {
                loadedModel.meshes.push(mesh);
                // leave meshes already parented to maintain model hierarchy:
                if (!mesh.parent) {
                    mesh.parent = loadedModel.rootMesh;
                }
                if (_this.props.receiveShadows === true) {
                    mesh.receiveShadows = true;
                }
            });
            loadedModel.particleSystems = particleSystems;
            loadedModel.skeletons = skeletons;
            loadedModel.animationGroups = animationGroups;
            if (_this.props.onModelLoaded) {
                _this.props.onModelLoaded(loadedModel);
            }
            loadedModel.status = LoaderStatus.Loaded;
            // we want to trigger after mesh is loaded (ie: position/rotation)
            var updates = new ModelPropsHandler().getPropertyUpdates({ rootUrl: "", sceneFilename: "" }, _this.props);
            if (updates !== null) {
                updates.forEach(function (update) { return applyUpdateToInstance(instance.hostInstance, update, "model"); });
            }
            if (_this.props.scaleToDimension && loadedModel && loadedModel.scaleToDimension !== _this.props.scaleToDimension) {
                var boundingInfo = loadedModel.boundingInfo; // will be null when no meshes are loaded
                if (boundingInfo) {
                    var longestDimension = Math.max(Math.abs(boundingInfo.minimum.x - boundingInfo.maximum.x), Math.abs(boundingInfo.minimum.y - boundingInfo.maximum.y), Math.abs(boundingInfo.minimum.z - boundingInfo.maximum.z));
                    var dimension = _this.props.scaleToDimension / longestDimension;
                    loadedModel.rootMesh.scaling.scaleInPlace(dimension);
                    loadedModel.scaleToDimension = _this.props.scaleToDimension;
                }
            }
        }, function (event) {
            if (_this.props.onLoadProgress) {
                _this.props.onLoadProgress(event);
            }
        }, function (scene, message, exception) {
            loadedModel.status = LoaderStatus.Error;
            loadedModel.errorMessage = "error: " + message + " -> " + (exception ? exception.message : "no exception");
            if (_this.props.onModelError) {
                _this.props.onModelError(loadedModel);
            }
        }, this.props.pluginExtension);
        if (loader) {
            loadedModel.loaderName = loader.name;
        }
        else {
            loadedModel.loaderName = "no loader found";
        }
    };
    ModelLifecycleListener.prototype.onUnmount = function () { };
    return ModelLifecycleListener;
}());
export default ModelLifecycleListener;
//# sourceMappingURL=ModelLifecycleListener.js.map