import { useContext, useEffect, useRef, useState } from 'react';
import { ActionManager, ExecuteCodeAction, } from '@babylonjs/core';
import { SceneContext } from './Scene';
import { CustomPropsHandler } from './PropsHandler';
export function useBeforeRender(callback, mask, insertFirst, callOnce) {
    var _a = useContext(SceneContext), scene = _a.scene, sceneReady = _a.sceneReady;
    useEffect(function () {
        if (sceneReady !== true || scene === null) {
            return;
        }
        var unregisterOnFirstCall = callOnce === true;
        var sceneObserver = scene.onBeforeRenderObservable.add(callback, mask, insertFirst, undefined, unregisterOnFirstCall);
        if (unregisterOnFirstCall !== true) {
            return function () {
                scene.onBeforeRenderObservable.remove(sceneObserver);
            };
        }
    });
}
export function useAfterRender(callback, mask, insertFirst, callOnce) {
    var _a = useContext(SceneContext), scene = _a.scene, sceneReady = _a.sceneReady;
    useEffect(function () {
        if (sceneReady !== true || scene === null) {
            return;
        }
        var unregisterOnFirstCall = callOnce === true;
        var sceneObserver = scene.onAfterRenderObservable.add(callback, mask, insertFirst, undefined, unregisterOnFirstCall);
        if (unregisterOnFirstCall !== true) {
            return function () {
                scene.onAfterRenderObservable.remove(sceneObserver);
            };
        }
    });
}
export var useCustomPropsHandler = function (propsHandler /*, deps?: React.DependencyList | undefined*/) {
    var firstRun = useRef(true);
    if (firstRun.current === true) {
        CustomPropsHandler.RegisterPropsHandler(propsHandler);
        firstRun.current = false;
    }
    useEffect(function () {
        return function () {
            // console.warn('de-registering on unmount', propsHandler.name);
            CustomPropsHandler.UnregisterPropsHandler(propsHandler);
        };
    }, []);
};
/**
 * useHover hook
 *
 * TODO: support GUI 3D contols
 *
 * @param over expression when hover over starts
 * @param out expression when hover stops
 */
export var useHover = function (over, out) {
    var _a = useState(false), value = _a[0], setValue = _a[1];
    var ref = useRef(null);
    useEffect(function () {
        if (ref.current) {
            var registeredMeshActions_1 = [];
            var observer2dGuiEnter_1 = null;
            var observer2dGuiOut_1 = null;
            if (ref.current.metadata.isMesh === true) {
                var mesh = ref.current.hostInstance;
                if (!mesh.actionManager) {
                    mesh.actionManager = new ActionManager(mesh.getScene());
                }
                var onPointerOverAction = mesh.actionManager.registerAction(new ExecuteCodeAction(ActionManager.OnPointerOverTrigger, function (ev) {
                    over && over(ev);
                    setValue(true);
                }));
                var onPointerOutAction = mesh.actionManager.registerAction(new ExecuteCodeAction(ActionManager.OnPointerOutTrigger, function (ev) {
                    out && out(ev);
                    setValue(false);
                }));
                registeredMeshActions_1.push(onPointerOverAction);
                registeredMeshActions_1.push(onPointerOutAction);
            }
            else if (ref.current.metadata.isGUI2DControl === true) {
                var control = ref.current.hostInstance;
                observer2dGuiEnter_1 = control.onPointerEnterObservable.add(over);
                observer2dGuiOut_1 = control.onPointerOutObservable.add(out);
            }
            else {
                console.warn("Can only apply useHover to non-mesh/2D control currently.", ref.current.metadata);
            }
            if (registeredMeshActions_1.length > 0 || observer2dGuiEnter_1 !== null) {
                return function () {
                    if (ref.current) {
                        if (registeredMeshActions_1.length > 0) {
                            registeredMeshActions_1.forEach(function (action) {
                                var _a;
                                if (action !== null) {
                                    var mesh = ref.current.hostInstance;
                                    (_a = mesh.actionManager) === null || _a === void 0 ? void 0 : _a.unregisterAction(action);
                                }
                            });
                            registeredMeshActions_1.splice(0, registeredMeshActions_1.length);
                        }
                        if (observer2dGuiEnter_1 !== null) {
                            var control = ref.current.hostInstance;
                            control.onPointerEnterObservable.remove(observer2dGuiEnter_1);
                            control.onPointerOutObservable.remove(observer2dGuiOut_1);
                            observer2dGuiEnter_1 = null;
                            observer2dGuiOut_1 = null;
                        }
                    }
                };
            }
        }
    }, [ref.current]);
    // todo: if use ref.current as dep,  duplicate register action.
    return [ref, value];
};
/**
 * useClick hook
 *
 * TODO: support UI
 * @param onClick
 */
export function useClick(onClick) {
    var ref = useRef(null);
    useEffect(function () {
        if (ref.current) {
            if (ref.current.metadata.isMesh === true) {
                var mesh = ref.current.hostInstance;
                if (!mesh.actionManager) {
                    mesh.actionManager = new ActionManager(mesh.getScene());
                }
                mesh.actionManager.registerAction(new ExecuteCodeAction(ActionManager.OnPickTrigger, function (ev) {
                    onClick(ev);
                }));
            }
            else {
                console.warn('onClick hook only supports referencing Meshes');
            }
        }
    }, [ref]);
    // todo: if use ref.current as dep,  duplicate register action.
    return [ref];
}
//# sourceMappingURL=hooks.js.map