import { useEffect, useState } from "react";
import "@babylonjs/loaders";
import { SceneLoader } from "@babylonjs/core";
import { useBabylonScene } from "../Scene";
var importMesh = function (meshNames, rootUrl, sceneFilename, pluginExtension, scene) {
    return new Promise(function (resolve, reject) {
        SceneLoader.ImportMesh(meshNames, rootUrl, sceneFilename, scene, function (meshes, particleSystems, skeletons, animationGroups) {
            resolve({ meshes: meshes, particleSystems: particleSystems, skeletons: skeletons, animationGroups: animationGroups });
        }, undefined, function (_, message, exception) {
            reject(exception !== null && exception !== void 0 ? exception : message);
        }, pluginExtension);
    });
};
/**
 * Same parameters as SceneLoader.ImportMesh (see docs)
 *
 * @param rootUrl
 * @param sceneFilename
 * @param pluginExtension
 * @param meshNames
 */
export var useLoader = function (rootUrl, sceneFilename, pluginExtension, meshNames) {
    var scene = useBabylonScene();
    var _a = useState(false), loaded = _a[0], setLoaded = _a[1];
    // using empty object to allow destructuring from hook (instead of Nullable<LoaderResult>)
    var _b = useState({
        animationGroups: [],
        meshes: [],
        particleSystems: [],
        skeletons: []
    }), results = _b[0], setResults = _b[1];
    useEffect(function () {
        importMesh(meshNames, rootUrl, sceneFilename, pluginExtension, scene)
            .then(function (result) {
            setResults(result);
            setLoaded(true);
        });
    }, [rootUrl, sceneFilename]);
    return [loaded, results];
};
// TODO: add support for Suspense
// type AsyncStatus = 'init' | 'success' | 'error';
// let status: AsyncStatus = 'init';
// let loadedModel = {meshes: [], ...}
// const suspender = importMesh(...)
//         .then((loaded: LoadedModel) => {
//             loadedModel = loaded; // available from closure
//             status = 'success';
//         })
//         .catch((e) => {
//             error = e;
//             status = 'error';
//         });
//     return () => {
//         if (status === 'init') {
//             // Yes, throw the Promise if it's not resolved yet!
//             throw suspender;
//         } else if (status === 'error') {
//             throw error;
//         } else {
//             return loadedModel;
//         }
//     }
//# sourceMappingURL=useLoader.js.map